﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Gma.UserActivityMonitor;

namespace ScreenMouseRotate
{
    public partial class Form1 : Form
    {
        private string[] orientationNames = new string[4] { "Default", "90", "180", "270" };
        private int[] orientationValues = new int[4]{NativeMethods.DMDO_DEFAULT,
														NativeMethods.DMDO_90,
														NativeMethods.DMDO_180,
														NativeMethods.DMDO_270};
        private int orientation = 0;        // Default
        private int SCREENWIDTH = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width;
        private int SCREENHEIGHT = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height;
        private int lastX = -1, lastY = -1;

        public Form1()
        {
            InitializeComponent();
            // Activate mouse hook
            HookManager.MouseMove += HookManager_MouseMove;
        }

        private void HookManager_MouseMove(object sender, MouseEventArgs e)
        {
            //Console.WriteLine(string.Format("x={0:0000}; y={1:0000}", e.X, e.Y));
            //// If a pen is detected, don't do any translation.  The pen is in the right place.
            // Just remember the coordinates when we switch back to the mouse
            if (e.Clicks < 0)
            {
                System.Windows.Forms.Cursor.Position = new Point(e.X, e.Y);
                lastX = e.X;
                lastY = e.Y;
                return;
            }
            if ((e.X != lastX) || (e.Y != lastY))
            {
                if (orientation == 2)           // Rotated 90 degrees
                {
                    if (lastX == -1)    // Touch or pen detected
                    {
                        lastX = e.X;
                        lastY = e.Y;
                        System.Windows.Forms.Cursor.Position = new Point(e.X, e.Y);
                    }
                    else
                    {
                        int diffX = lastX - e.X;
                        int diffY = lastY - e.Y;
                        int newX = lastX - diffY;
                        if (newX < 0) newX = 0;
                        if (newX >= this.SCREENHEIGHT) newX = this.SCREENHEIGHT - 1;
                        int newY = lastY + diffX;
                        if (newY < 0) newY = 0;
                        if (newY >= this.SCREENWIDTH) newY = this.SCREENWIDTH - 1;
                        System.Windows.Forms.Cursor.Position = new Point(newX, newY);
                        lastX = newX;
                        lastY = newY;
                    }
                }
                else if (orientation == 0)
                {
                    // Should never be here?
                    System.Windows.Forms.Cursor.Position = new Point(e.X, e.Y);
                    lastX = e.X;
                    lastY = e.Y;
                }
                else if (orientation == 1)
                {
                    if (lastX == -1)        // Touch or pen detected
                    {
                        lastX = e.X;
                        lastY = e.Y;
                        System.Windows.Forms.Cursor.Position = new Point(e.X, e.Y);
                    }
                    else
                    {
                        int diffX = e.X - lastX;
                        int diffY = e.Y - lastY;
                        int newX = lastX - diffY;
                        if (newX < 0) newX = 0;
                        if (newX >= this.SCREENHEIGHT) newX = this.SCREENHEIGHT - 1;
                        int newY = lastY + diffX;
                        if (newY < 0) newY = 0;
                        if (newY >= this.SCREENWIDTH) newY = this.SCREENWIDTH - 1;
                        System.Windows.Forms.Cursor.Position = new Point(newX, newY);
                        lastX = newX;
                        lastY = newY;
                    }
                }
                else
                {
                    // Error condition
                    MessageBox.Show("Orientation code error, ScreenMouseRotate exiting.");
                    this.Close();
                }
            }
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == WindowState)
                Hide();
        }

        private void notifyIcon1_DoubleClick(object sender, EventArgs e)
        {
            Show();
            WindowState = FormWindowState.Normal;
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(linkLabel1.Text);
        }

        private void linkLabel2_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(linkLabel2.Text);
        }


        private void linkLabel3_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("mailto:" + linkLabel3.Text);
        }

        private void btnDefault_Click(object sender, EventArgs e)
        {
            Gma.UserActivityMonitor.HookManager.rotated = false;
            orientation = 0;
            rotate0();
            lastX = System.Windows.Forms.Cursor.Position.X;
            lastY = System.Windows.Forms.Cursor.Position.Y;  
        }

        private void btnPortrait1_Click(object sender, EventArgs e)
        {
            Gma.UserActivityMonitor.HookManager.rotated = true;
            orientation = 1;
            rotate90();
            lastX = System.Windows.Forms.Cursor.Position.X;
            lastY = System.Windows.Forms.Cursor.Position.Y;  
        }

        private void btnPortrait2_Click(object sender, EventArgs e)
        {
            Gma.UserActivityMonitor.HookManager.rotated = true;
            orientation = 2;
            rotate270();
            lastX = System.Windows.Forms.Cursor.Position.X;
            lastY = System.Windows.Forms.Cursor.Position.Y;  
        }

        /*
         * Rotate code
         */
        private void rotate90()
        {
            // obtain current settings
            DEVMODE dm = NativeMethods.CreateDevmode();
            GetSettings(ref dm);

            dm.dmPelsHeight = SCREENWIDTH;
            dm.dmPelsWidth = SCREENHEIGHT;

            // set the orientation value             
            dm.dmDisplayOrientation = NativeMethods.DMDO_90;

            // switch to new settings
            ChangeSettings(dm);
        }

        private void rotate270()
        {
            // obtain current settings
            DEVMODE dm = NativeMethods.CreateDevmode();
            GetSettings(ref dm);

            dm.dmPelsHeight = SCREENWIDTH;
            dm.dmPelsWidth = SCREENHEIGHT;

            // set the orientation value             
            dm.dmDisplayOrientation = NativeMethods.DMDO_270;

            // switch to new settings
            ChangeSettings(dm);
        }

        private void rotate0()
        {
            // obtain current settings
            DEVMODE dm = NativeMethods.CreateDevmode();
            GetSettings(ref dm);

            dm.dmPelsHeight = SCREENHEIGHT;
            dm.dmPelsWidth = SCREENWIDTH;

            // set the orientation value
            dm.dmDisplayOrientation = NativeMethods.DMDO_DEFAULT;

            // switch to new settings
            ChangeSettings(dm);
        }

        private void ChangeSettings(DEVMODE dm)
        {
            // helper to wrap ChangeDisplaySettings Win32 API
            int iRet = NativeMethods.ChangeDisplaySettings(ref dm, 0);
            switch (iRet)
            {
                case NativeMethods.DISP_CHANGE_SUCCESSFUL:
                    break;
                case NativeMethods.DISP_CHANGE_RESTART:
                    MessageBox.Show("Please restart your system");
                    break;
                case NativeMethods.DISP_CHANGE_FAILED:
                    MessageBox.Show("ChangeDisplaySettigns API failed");
                    break;
                case NativeMethods.DISP_CHANGE_BADDUALVIEW:
                    MessageBox.Show("The settings change was unsuccessful because system is DualView capable.");
                    break;
                case NativeMethods.DISP_CHANGE_BADFLAGS:
                    MessageBox.Show("An invalid set of flags was passed in.");
                    break;
                case NativeMethods.DISP_CHANGE_BADPARAM:
                    MessageBox.Show("An invalid parameter was passed in. This can include an invalid flag or combination of flags.");
                    break;
                case NativeMethods.DISP_CHANGE_NOTUPDATED:
                    MessageBox.Show("Unable to write settings to the registry.");
                    break;
                default:
                    MessageBox.Show("Unknown return value from ChangeDisplaySettings API");
                    break;
            }
        }

        private int GetSettings(ref DEVMODE dm)
        {
            // helper to obtain current settings
            return GetSettings(ref dm, NativeMethods.ENUM_CURRENT_SETTINGS);
        }

        private int GetSettings(ref DEVMODE dm, int iModeNum)
        {
            // helper to wrap EnumDisplaySettings Win32 API
            return NativeMethods.EnumDisplaySettings(null, iModeNum, ref dm);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            HookManager.MouseMove -= HookManager_MouseMove;
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {

        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Show();
            WindowState = FormWindowState.Normal;
        }

        private void landscapeDefaultToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnDefault_Click(sender, e);
        }

        private void portrait1ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnPortrait1_Click(sender, e);
        }

        private void portrait2ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            btnPortrait2_Click(sender, e);
        }



    }
}
