//-------------------------------------------------------------------------- 
// 
//  Copyright (c) Microsoft Corporation.  All rights reserved. 
// 
//  File: NativeMethods.cs 
//			C# file for the DisplaySettingsSample application
// 
//-------------------------------------------------------------------------- 
using System;
using System.Runtime.InteropServices;

namespace ScreenMouseRotate
{
	[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	public struct DEVMODE 
	{
		[MarshalAs(UnmanagedType.ByValTStr,SizeConst=32)]
		public string dmDeviceName;
		
		public short  dmSpecVersion;
		public short  dmDriverVersion;
		public short  dmSize;
		public short  dmDriverExtra;
		public int    dmFields;
		public int    dmPositionX;
		public int    dmPositionY;
		public int    dmDisplayOrientation;
		public int    dmDisplayFixedOutput;
		public short  dmColor;
		public short  dmDuplex;
		public short  dmYResolution;
		public short  dmTTOption;
		public short  dmCollate;

		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
		public string dmFormName;

		public short  dmLogPixels;
		public short  dmBitsPerPel;
		public int    dmPelsWidth;
		public int    dmPelsHeight;
		public int    dmDisplayFlags;
		public int    dmDisplayFrequency;
		public int    dmICMMethod;
		public int    dmICMIntent;
		public int    dmMediaType;
		public int    dmDitherType;
		public int    dmReserved1;
		public int    dmReserved2;
		public int    dmPanningWidth;
		public int    dmPanningHeight;
	};

	public class NativeMethods
	{
		// PInvoke declaration for EnumDisplaySettings Win32 API
		[DllImport("user32.dll", CharSet=CharSet.Ansi)]
		public static extern int EnumDisplaySettings(string lpszDeviceName, int iModeNum, ref DEVMODE lpDevMode);         

		// PInvoke declaration for ChangeDisplaySettings Win32 API
		[DllImport("user32.dll", CharSet=CharSet.Ansi)]
		public static extern int ChangeDisplaySettings(ref DEVMODE lpDevMode, int dwFlags);

		// helper for creating an initialized DEVMODE structure
		public static DEVMODE CreateDevmode()
		{
			DEVMODE dm = new DEVMODE();
			dm.dmDeviceName = new String(new char[32]);
			dm.dmFormName = new String(new char[32]);
			dm.dmSize = (short)Marshal.SizeOf(dm);
			return dm;
		}

		// constants
		public const int ENUM_CURRENT_SETTINGS = -1;
		public const int DISP_CHANGE_SUCCESSFUL = 0;
		public const int DISP_CHANGE_BADDUALVIEW = -6;
		public const int DISP_CHANGE_BADFLAGS = -4;
		public const int DISP_CHANGE_BADMODE = -2;
		public const int DISP_CHANGE_BADPARAM = -5;
		public const int DISP_CHANGE_FAILED = -1;
		public const int DISP_CHANGE_NOTUPDATED = -3;
		public const int DISP_CHANGE_RESTART = 1;
		public const int DMDO_DEFAULT = 0;
		public const int DMDO_90 = 1;
		public const int DMDO_180 = 2;
		public const int DMDO_270 = 3;
	}
}
