using System;
using System.Drawing;
using System.Windows.Forms;
using Gma.UserActivityMonitor;
using System.ComponentModel;
using Microsoft.Win32;

namespace Gma.UserActivityMonitorDemo
{
    public partial class TestFormStatic : Form
    {
        private string[] orientationNames = new string[4] { "Default", "90", "180", "270" };
        private int[] orientationValues = new int[4]{NativeMethods.DMDO_DEFAULT,
														NativeMethods.DMDO_90,
														NativeMethods.DMDO_180,
														NativeMethods.DMDO_270};
        private int orientation = 0;        // Default
        private int SCREENWIDTH = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width;
        private int SCREENHEIGHT = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height;
        private int lastX = -1, lastY = -1;
  

        public TestFormStatic()
        {
            InitializeComponent();
            DEVMODE dm = NativeMethods.CreateDevmode();
            // Activate mouse hook
            HookManager.MouseMove += HookManager_MouseMove;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

        //##################################################################
        #region Check boxes to set or remove particular event handlers.

        private void checkBoxOnMouseMove_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxOnMouseMove.Checked)
            {
                HookManager.MouseMove += HookManager_MouseMove;
            }
            else
            {
                HookManager.MouseMove -= HookManager_MouseMove;
            }
        }

        private void checkBoxOnMouseClick_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxOnMouseClick.Checked)
            {
                HookManager.MouseClick += HookManager_MouseClick;
            }
            else
            {
                HookManager.MouseClick -= HookManager_MouseClick;
            }
        }

        private void checkBoxOnMouseUp_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxOnMouseUp.Checked)
            {
                HookManager.MouseUp += HookManager_MouseUp;
            }
            else
            {
                HookManager.MouseUp -= HookManager_MouseUp;
            }
        }

        private void checkBoxOnMouseDown_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxOnMouseDown.Checked)
            {
                HookManager.MouseDown += HookManager_MouseDown;
            }
            else
            {
                HookManager.MouseDown -= HookManager_MouseDown;
            }
        }

        private void checkBoxMouseDoubleClick_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxMouseDoubleClick.Checked)
            {
                HookManager.MouseDoubleClick += HookManager_MouseDoubleClick;
            }
            else
            {
                HookManager.MouseDoubleClick -= HookManager_MouseDoubleClick;
            }
        }

        private void checkBoxMouseWheel_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxMouseWheel.Checked)
            {
                HookManager.MouseWheel += HookManager_MouseWheel;
            }
            else
            {
                HookManager.MouseWheel -= HookManager_MouseWheel;
            }
        }

        private void checkBoxKeyDown_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxKeyDown.Checked)
            {
                HookManager.KeyDown += HookManager_KeyDown;
            }
            else
            {
                HookManager.KeyDown -= HookManager_KeyDown;
            }
        }


        private void checkBoxKeyUp_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxKeyUp.Checked)
            {
                HookManager.KeyUp += HookManager_KeyUp;
            }
            else
            {
                HookManager.KeyUp -= HookManager_KeyUp;
            }
        }

        private void checkBoxKeyPress_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxKeyPress.Checked)
            {
                HookManager.KeyPress += HookManager_KeyPress;
            }
            else
            {
                HookManager.KeyPress -= HookManager_KeyPress;
            }
        }

        #endregion

        //##################################################################
        #region Event handlers of particular events. They will be activated when an appropriate checkbox is checked.


        private void HookManager_MouseMove(object sender, MouseEventArgs e)
        {
            labelMousePosition.Text = string.Format("x={0:0000}; y={1:0000}", e.X, e.Y);
            //// If the pen is detected, don't do any translation.  The pen is in the right place.
            // Just remember the coordinates when we switch back to the mouse
            if (e.Clicks < 0)
            {
                System.Windows.Forms.Cursor.Position = new Point(e.X, e.Y);
                lastX = e.X;
                lastY = e.Y;
                return;
            }
            if ((e.X != lastX) || (e.Y != lastY))
            {
                if (orientation == 1)           // Rotated 90 degrees
                {
                    if (lastX == -1)
                    {
                        lastX = e.X;
                        lastY = e.Y;
                        System.Windows.Forms.Cursor.Position = new Point(e.X, e.Y);
                    }
                    else
                    {
                        int diffX = lastX - e.X;
                        int diffY = lastY - e.Y;
                        int newX = lastX - diffY;
                        if (newX < 0) newX = 0;
                        if (newX >= this.SCREENHEIGHT) newX = this.SCREENHEIGHT - 1;
                        int newY = lastY + diffX;
                        if (newY < 0) newY = 0;
                        if (newY >= this.SCREENWIDTH) newY = this.SCREENWIDTH - 1;
                        System.Windows.Forms.Cursor.Position = new Point(newX, newY);
                        lastX = newX;
                        lastY = newY;
                    }
                }
            }
        }

        private void HookManager_KeyDown(object sender, KeyEventArgs e)
        {
            textBoxLog.AppendText(string.Format("KeyDown - {0}\n", e.KeyCode));
            textBoxLog.ScrollToCaret();
        }

        private void HookManager_KeyUp(object sender, KeyEventArgs e)
        {
            textBoxLog.AppendText(string.Format("KeyUp - {0}\n", e.KeyCode));
            textBoxLog.ScrollToCaret();
        }


        private void HookManager_KeyPress(object sender, KeyPressEventArgs e)
        {
            textBoxLog.AppendText(string.Format("KeyPress - {0}\n", e.KeyChar));
            textBoxLog.ScrollToCaret();
        } 

        private void HookManager_MouseClick(object sender, MouseEventArgs e)
        {
            textBoxLog.AppendText(string.Format("MouseClick - {0}\n", e.Button));
            textBoxLog.ScrollToCaret();
        }


        private void HookManager_MouseUp(object sender, MouseEventArgs e)
        {
            textBoxLog.AppendText(string.Format("MouseUp - {0}\n", e.Button));
            textBoxLog.ScrollToCaret();
        }


        private void HookManager_MouseDown(object sender, MouseEventArgs e)
        {
            textBoxLog.AppendText(string.Format("MouseDown - {0}\n", e.Button));
            textBoxLog.ScrollToCaret();
        }


        private void HookManager_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            textBoxLog.AppendText(string.Format("MouseDoubleClick - {0}\n", e.Button));
            textBoxLog.ScrollToCaret();
        }


        private void HookManager_MouseWheel(object sender, MouseEventArgs e)
        {
            labelWheel.Text = string.Format("Wheel={0:000}", e.Delta);
        }

        #endregion

  
        private void button1_Click(object sender, EventArgs e)
        {
            Gma.UserActivityMonitor.HookManager.rotated = !Gma.UserActivityMonitor.HookManager.rotated;
            if (orientation == 0)
            {
                orientation = 1;        // Flip to 90
                rotate90();
            }
            else if (orientation == 1)
            {
                orientation = 0;        // Flip to 0
                rotate0();
            }
            lastX = System.Windows.Forms.Cursor.Position.X;
            lastY = System.Windows.Forms.Cursor.Position.Y;            
        }



        private void rotate90()
        {
            // obtain current settings
            DEVMODE dm = NativeMethods.CreateDevmode();
            GetSettings(ref dm);
           
            dm.dmPelsHeight = SCREENWIDTH;
            dm.dmPelsWidth = SCREENHEIGHT;

            // set the orientation value             
            dm.dmDisplayOrientation = NativeMethods.DMDO_270;

            // switch to new settings
            ChangeSettings(dm);
        }

        private void rotate0()
        {
            // obtain current settings
            DEVMODE dm = NativeMethods.CreateDevmode();
            GetSettings(ref dm);

            dm.dmPelsHeight = SCREENHEIGHT;
            dm.dmPelsWidth = SCREENWIDTH;

            // set the orientation value
            dm.dmDisplayOrientation = NativeMethods.DMDO_DEFAULT;

            // switch to new settings
            ChangeSettings(dm);
        }

        private void ChangeSettings(DEVMODE dm)
        {
            // helper to wrap ChangeDisplaySettings Win32 API
            int iRet = NativeMethods.ChangeDisplaySettings(ref dm, 0);
            switch (iRet)
            {
                case NativeMethods.DISP_CHANGE_SUCCESSFUL:
                    break;
                case NativeMethods.DISP_CHANGE_RESTART:
                    MessageBox.Show("Please restart your system");
                    break;
                case NativeMethods.DISP_CHANGE_FAILED:
                    MessageBox.Show("ChangeDisplaySettigns API failed");
                    break;
                case NativeMethods.DISP_CHANGE_BADDUALVIEW:
                    MessageBox.Show("The settings change was unsuccessful because system is DualView capable.");
                    break;
                case NativeMethods.DISP_CHANGE_BADFLAGS:
                    MessageBox.Show("An invalid set of flags was passed in.");
                    break;
                case NativeMethods.DISP_CHANGE_BADPARAM:
                    MessageBox.Show("An invalid parameter was passed in. This can include an invalid flag or combination of flags.");
                    break;
                case NativeMethods.DISP_CHANGE_NOTUPDATED:
                    MessageBox.Show("Unable to write settings to the registry.");
                    break;
                default:
                    MessageBox.Show("Unknown return value from ChangeDisplaySettings API");
                    break;
            }            
        }

        private int GetSettings(ref DEVMODE dm)
        {
            // helper to obtain current settings
            return GetSettings(ref dm, NativeMethods.ENUM_CURRENT_SETTINGS);
        }

        private int GetSettings(ref DEVMODE dm, int iModeNum)
        {
            // helper to wrap EnumDisplaySettings Win32 API
            return NativeMethods.EnumDisplaySettings(null, iModeNum, ref dm);
        }


    }
}